@@ -82,50 +82,82 @@ public class DefaultRepositoryMetadataManager
                                           localRepository.pathOfLocalRepositoryMetadata( metadata, repository ) );
 
 
-
                     boolean checkForUpdates =
-                        policy.checkOutOfDate( new Date( file.lastModified() ) ) || !file.exists();
-
-                    boolean metadataIsEmpty = true;
+                        !file.exists() || policy.checkOutOfDate( new Date( file.lastModified() ) );
 
                     if ( checkForUpdates )
                     {
-                        getLogger().info( metadata.getKey() + ": checking for updates from " + repository.getId() );
+                        if ( wagonManager.isOnline() )
+                        {
+                            getLogger().info( metadata.getKey() + ": checking for updates from " + repository.getId() );
 
-                        try
-                        {
-                            resolveAlways( metadata, repository, file, policy.getChecksumPolicy(), true );
-                            metadataIsEmpty = false;
-                        }
-                        catch ( TransferFailedException e )
-                        {
-                            // TODO: [jc; 08-Nov-2005] revisit this for 2.1
-                            // suppressing logging to avoid logging this error twice.
-                            metadataIsEmpty = true;
-                        }
-                    }
+                            boolean storeMetadata = false;
+                            try
+                            {
+                                wagonManager.getArtifactMetadata( metadata, repository, file,
+                                                                  policy.getChecksumPolicy() );
+                                storeMetadata = true;
+                            }
+                            catch ( ResourceDoesNotExistException e )
+                            {
+                                getLogger().debug(
+                                    metadata + " could not be found on repository: " + repository.getId() );
 
-                    // TODO: should this be inside the above check?
-                    // touch file so that this is not checked again until interval has passed
-                    if ( file.exists() )
-                    {
-                        file.setLastModified( System.currentTimeMillis() );
-                    }
-                    else
-                    {
-                        // this ensures that files are not continuously checked when they don't exist remotely
-                        try
-                        {
-                            metadata.storeInLocalRepository( localRepository, repository );
+                                // delete the local copy so the old details aren't used.
+                                if ( file.exists() )
+                                {
+                                    file.delete();
+                                }
+                                storeMetadata = true;
+                            }
+                            catch ( TransferFailedException e )
+                            {
+                                getLogger().warn( metadata + " could not be retrieved from repository: " +
+                                    repository.getId() + " due to an error: " + e.getMessage() );
+                                getLogger().debug( "Exception", e );
+
+                                getLogger().info( "Repository '" + repository.getId() + "' will be blacklisted" );
+                                repository.setBlacklisted( true );
+
+                                // TODO: [jc; 08-Nov-2005] revisit this for 2.1
+                                // suppressing logging to avoid logging this error twice.
+                            }
+                            if ( storeMetadata )
+                            {
+                                // touch file so that this is not checked again until interval has passed
+                                if ( file.exists() )
+                                {
+                                    file.setLastModified( System.currentTimeMillis() );
+                                }
+                                else
+                                {
+                                    // this ensures that files are not continuously checked when they don't exist remotely
+
+                                    // TODO: [jdcasey] If this happens as a result of ResourceDoesNotExistException, what effect will it have on subsequent runs?
+                                    // Will the updateInterval come into play cleanly, or will this plug up the works??
+                                    try
+                                    {
+                                        metadata.storeInLocalRepository( localRepository, repository );
+                                    }
+                                    catch ( RepositoryMetadataStoreException e )
+                                    {
+                                        throw new RepositoryMetadataResolutionException(
+                                            "Unable to store local copy of metadata: " + e.getMessage(), e );
+                                    }
+                                }
+                            }
                         }
-                        catch ( RepositoryMetadataStoreException e )
+                        else
                         {
-                            throw new RepositoryMetadataResolutionException(
-                                "Unable to store local copy of metadata: " + e.getMessage(), e );
+                            getLogger().debug( "System is offline. Cannot resolve metadata:\n" +
+                                metadata.extendedToString() + "\n\n" );
                         }
                     }
                 }
             }
+
+            // TODO: [jdcasey] what happens here when the system is offline, or there is a TransferFailedException
+            // ...and no metadata file is written?
             cachedMetadata.add( metadata.getKey() );
         }
 
@@ -216,7 +248,7 @@ public class DefaultRepositoryMetadataManager
                 }
                 else
                 {
-                    if ( m.getVersioning() != null && m.getVersioning().getSnapshot() != null &&
+                    if ( ( m.getVersioning() != null ) && ( m.getVersioning().getSnapshot() != null ) &&
                         m.getVersioning().getSnapshot().isLocalCopy() )
                     {
                         m.getVersioning().getSnapshot().setLocalCopy( false );
@@ -243,7 +275,7 @@ public class DefaultRepositoryMetadataManager
         {
             Metadata metadata = readMetadata( metadataFile );
 
-            if ( repoMetadata.isSnapshot() && previousMetadata != null )
+            if ( repoMetadata.isSnapshot() && ( previousMetadata != null ) )
             {
                 previousMetadata.put( remoteRepository, metadata );
             }
@@ -310,19 +342,15 @@ public class DefaultRepositoryMetadataManager
                 "System is offline. Cannot resolve required metadata:\n" + metadata.extendedToString() );
         }
 
-        File file = new File( localRepository.getBasedir(),
-                              localRepository.pathOfLocalRepositoryMetadata( metadata, remoteRepository ) );
-
+        File file;
         try
         {
-            resolveAlways( metadata, remoteRepository, file, ArtifactRepositoryPolicy.CHECKSUM_POLICY_WARN, false );
+            file = getArtifactMetadataFromDeploymentRepository( metadata, localRepository, remoteRepository );
         }
         catch ( TransferFailedException e )
         {
-            // TODO: [jc; 08-Nov-2005] revisit this for 2.1
-            // suppressing logging to avoid logging this error twice.
-            // We don't want to interrupt program flow here. Just allow empty metadata instead.
-            // rethrowing this would change behavior.
+            throw new RepositoryMetadataResolutionException( metadata + " could not be retrieved from repository: " +
+                remoteRepository.getId() + " due to an error: " + e.getMessage(), e );
         }
 
         try
@@ -339,33 +367,23 @@ public class DefaultRepositoryMetadataManager
         }
     }
 
-    private void resolveAlways( ArtifactMetadata metadata, ArtifactRepository repository, File file,
-                                String checksumPolicy, boolean allowBlacklisting )
-        throws RepositoryMetadataResolutionException, TransferFailedException
+    private File getArtifactMetadataFromDeploymentRepository( ArtifactMetadata metadata,
+                                                              ArtifactRepository localRepository,
+                                                              ArtifactRepository remoteRepository )
+        throws TransferFailedException
     {
-        if ( !wagonManager.isOnline() )
-        {
-            if ( allowBlacklisting )
-            {
-                getLogger().debug(
-                    "System is offline. Cannot resolve metadata:\n" + metadata.extendedToString() + "\n\n" );
-                return;
-            }
-            else
-            {
-                // metadata is required for deployment, can't be offline
-                throw new RepositoryMetadataResolutionException(
-                    "System is offline. Cannot resolve required metadata:\n" + metadata.extendedToString() );
-            }
-        }
+        File file = new File( localRepository.getBasedir(),
+                              localRepository.pathOfLocalRepositoryMetadata( metadata, remoteRepository ) );
 
         try
         {
-            wagonManager.getArtifactMetadata( metadata, repository, file, checksumPolicy );
+            wagonManager.getArtifactMetadataFromDeploymentRepository( metadata, remoteRepository, file,
+                                                                      ArtifactRepositoryPolicy.CHECKSUM_POLICY_WARN );
         }
         catch ( ResourceDoesNotExistException e )
         {
-            getLogger().debug( metadata + " could not be found on repository: " + repository.getId() );
+            getLogger().info(
+                metadata + " could not be found on repository: " + remoteRepository.getId() + ", so will be created" );
 
             // delete the local copy so the old details aren't used.
             if ( file.exists() )
@@ -373,16 +391,7 @@ public class DefaultRepositoryMetadataManager
                 file.delete();
             }
         }
-        catch ( TransferFailedException e )
-        {
-            getLogger().warn( metadata + " could not be retrieved from repository: " + repository.getId() +
-                " due to an error: " + e.getMessage() );
-            getLogger().info( "Repository '" + repository.getId() + "' will be blacklisted" );
-            getLogger().debug( "Exception", e );
-            repository.setBlacklisted( allowBlacklisting );
-
-            throw e;
-        }
+        return file;
     }
 
     private boolean alreadyResolved( ArtifactMetadata metadata )
@@ -401,26 +410,26 @@ public class DefaultRepositoryMetadataManager
                 "System is offline. Cannot deploy metadata:\n" + metadata.extendedToString() );
         }
 
-        getLogger().info( "Retrieving previous metadata from " + deploymentRepository.getId() );
-
-        File file = new File( localRepository.getBasedir(),
-                              localRepository.pathOfLocalRepositoryMetadata( metadata, deploymentRepository ) );
-
-        try
+        File file;
+        if ( metadata instanceof RepositoryMetadata )
         {
-            resolveAlways( metadata, deploymentRepository, file, ArtifactRepositoryPolicy.CHECKSUM_POLICY_WARN, false );
+            getLogger().info( "Retrieving previous metadata from " + deploymentRepository.getId() );
+            try
+            {
+                file = getArtifactMetadataFromDeploymentRepository( metadata, localRepository, deploymentRepository );
+            }
+            catch ( TransferFailedException e )
+            {
+                throw new RepositoryMetadataDeploymentException( metadata +
+                    " could not be retrieved from repository: " + deploymentRepository.getId() + " due to an error: " +
+                    e.getMessage(), e );
+            }
         }
-        catch ( RepositoryMetadataResolutionException e )
+        else
         {
-            throw new RepositoryMetadataDeploymentException(
-                "Unable to get previous metadata to update: " + e.getMessage(), e );
-        }
-        catch ( TransferFailedException e )
-        {
-            // TODO: [jc; 08-Nov-2005] revisit this for 2.1
-            // suppressing logging to avoid logging this error twice.
-            // We don't want to interrupt program flow here. Just allow empty metadata instead.
-            // rethrowing this would change behavior.
+            // It's a POM - we don't need to retrieve it first
+            file = new File( localRepository.getBasedir(),
+                             localRepository.pathOfLocalRepositoryMetadata( metadata, deploymentRepository ) );
         }
 
         try
