@@ -48,7 +48,7 @@ import org.codehaus.plexus.PlexusTestCase;
  * Test the default artifact collector.
  *
  * @author <a href="mailto:brett@apache.org">Brett Porter</a>
- * @version $Id: DefaultArtifactCollectorTest.java 592009 2007-11-05 14:24:53Z hboutemy $
+ * @version $Id: DefaultArtifactCollectorTest.java 640549 2008-03-24 20:05:11Z bentmann $
  */
 public class DefaultArtifactCollectorTest
     extends PlexusTestCase
@@ -285,6 +285,20 @@ public class DefaultArtifactCollectorTest
         assertEquals( "Check version", "2.0", getArtifact( "c", res.getArtifacts() ).getVersion() );
     }
 
+    public void testResolveRangeWithManagedVersion()
+        throws ArtifactResolutionException, InvalidVersionSpecificationException
+    {
+        ArtifactSpec a = createArtifactSpec( "a", "1.0" );
+        ArtifactSpec b = a.addDependency( "b", "[1.0,3.0]" );
+
+        ArtifactSpec managedB = createArtifactSpec( "b", "5.0" );
+
+        ArtifactResolutionResult res = collect( a, managedB.artifact );
+        assertEquals( "Check artifact list", createSet( new Object[] { a.artifact, managedB.artifact } ),
+                      res.getArtifacts() );
+        assertEquals( "Check version", "5.0", getArtifact( "b", res.getArtifacts() ).getVersion() );
+    }
+
     public void testCompatibleRanges()
         throws ArtifactResolutionException, InvalidVersionSpecificationException
     {
@@ -301,6 +315,110 @@ public class DefaultArtifactCollectorTest
         assertEquals( "Check version", "2.5", getArtifact( "c", res.getArtifacts() ).getVersion() );
     }
 
+    public void testCompatibleRecommendedVersion()
+        throws ArtifactResolutionException, InvalidVersionSpecificationException
+    {
+        
+        //this test puts two dependencies on C with 3.2 and [1.0,3.0] as the version.
+        //it puts 2.5 in the pretend repo...we should get back 2.5
+        ArtifactSpec a = createArtifactSpec( "a", "1.0" );
+        ArtifactSpec b = a.addDependency( "b", "1.0" );
+        ArtifactSpec b1 = a.addDependency( "b1", "1.0" );
+        b.addDependency( "c", "3.2" );
+        b1.addDependency( "c", "[1.0,3.0]" );
+      
+        //put it in the repo
+        ArtifactSpec c = createArtifactSpec( "c", "2.5" );
+        source.addArtifact( createArtifactSpec( "c", "2.5" ));
+
+        ArtifactResolutionResult res = collect( a );
+
+        assertEquals( "Check artifact list", createSet( new Object[] { a.artifact, b.artifact,b1.artifact,c.artifact } ),
+                      res.getArtifacts() );
+        assertEquals( "Check version", "2.5", getArtifact( "c", res.getArtifacts() ).getVersion() );
+    }
+    
+    public void testCompatibleRecommendedVersionWithChildren()
+        throws ArtifactResolutionException, InvalidVersionSpecificationException
+    {
+
+        // this test puts two dependencies on C with 3.2 and [1.0,3.0] as the version.
+        // it puts 2.5 in the pretend repo...we should get back c2.5 and d1.0
+        ArtifactSpec a = createArtifactSpec( "a", "1.0" );
+        ArtifactSpec b = a.addDependency( "b", "1.0" );
+        ArtifactSpec e = a.addDependency( "e", "1.0" );
+        ArtifactSpec c1 = b.addDependency( "c", "3.2" );
+        ArtifactSpec d1 = c1.addDependency( "d","1.1" );
+        e.addDependency( "c", "[1.0,3.0]" );
+
+        // put it in the repo
+        ArtifactSpec c = createArtifactSpec( "c", "2.5" );
+        ArtifactSpec d = c.addDependency( "d","1.0" );
+        
+        source.addArtifact( c );
+        source.addArtifact( d );
+        source.addArtifact( c1 );
+        source.addArtifact( d1 );
+
+        ArtifactResolutionResult res = collect( a );
+
+        assertEquals( "Check artifact list",
+                      createSet( new Object[] { a.artifact, b.artifact, e.artifact, c.artifact,d.artifact } ), res.getArtifacts() );
+        assertEquals( "Check version", "2.5", getArtifact( "c", res.getArtifacts() ).getVersion() );
+    }
+    
+    public void testInCompatibleRecommendedVersion()
+        throws ArtifactResolutionException, InvalidVersionSpecificationException
+    {
+
+        // this test puts two dependencies on C with 3.2 and [1.0,3.0] as the version.
+        // it puts 2.5 and 3.0 in the pretend repo...we should get back 3.0
+        ArtifactSpec a = createArtifactSpec( "a", "1.0" );
+        ArtifactSpec b = a.addDependency( "b", "1.0" );
+        ArtifactSpec b1 = a.addDependency( "b1", "1.0" );
+        b.addDependency( "c", "3.2" );
+        b1.addDependency( "c", "[1.0,3.0]" );
+
+        // put it in the repo
+        ArtifactSpec c = createArtifactSpec( "c", "3.0" );
+        source.addArtifact( createArtifactSpec( "c", "2.5" ) );
+        source.addArtifact( createArtifactSpec( "c", "3.0" ) );
+        source.addArtifact( createArtifactSpec( "c", "3.2" ) );
+
+        ArtifactResolutionResult res = collect( a );
+
+        assertEquals( "Check artifact list",
+                      createSet( new Object[] { a.artifact, b.artifact, b1.artifact, c.artifact } ), res.getArtifacts() );
+        assertEquals( "Check version", "3.0", getArtifact( "c", res.getArtifacts() ).getVersion() );
+    }
+
+    //MNG-2123: this version of the test caused the crash seen in the IT. It was
+    //only happening if the first dependency was not a range but the second one was.
+    public void testInCompatibleRecommendedVersion2()
+        throws ArtifactResolutionException, InvalidVersionSpecificationException
+    {
+
+        // this test puts two dependencies on C with 3.2 and [1.0,3.0] as the version.
+        // it puts 2.5 and 3.0 in the pretend repo...we should get back 3.0
+        ArtifactSpec a = createArtifactSpec( "a", "1.0" );
+        ArtifactSpec b = a.addDependency( "b", "1.0" );
+        ArtifactSpec b1 = a.addDependency( "b1", "1.0" );
+        b1.addDependency( "c", "3.2" );
+        b.addDependency( "c", "[1.0,3.0]" );
+
+        // put it in the repo
+        ArtifactSpec c = createArtifactSpec( "c", "3.0" );
+        source.addArtifact( createArtifactSpec( "c", "2.5" ) );
+        source.addArtifact( createArtifactSpec( "c", "3.0" ) );
+        source.addArtifact( createArtifactSpec( "c", "3.2" ) );
+
+        ArtifactResolutionResult res = collect( a );
+
+        assertEquals( "Check artifact list",
+                      createSet( new Object[] { a.artifact, b.artifact, b1.artifact, c.artifact } ), res.getArtifacts() );
+        assertEquals( "Check version", "3.0", getArtifact( "c", res.getArtifacts() ).getVersion() );
+    }
+    
     public void testIncompatibleRanges()
         throws ArtifactResolutionException, InvalidVersionSpecificationException
     {
@@ -843,8 +961,8 @@ public class DefaultArtifactCollectorTest
 
         private String getKey( Artifact artifact )
         {
-            //return artifact.getDependencyConflictId() + ":" + artifact.getVersionRange();
-            return artifact.getDependencyConflictId();
+            return artifact.getDependencyConflictId() + ":" + artifact.getVersion();
+            //return artifact.getDependencyConflictId();
         }
 
         private Set createArtifacts( ArtifactFactory artifactFactory, Set dependencies, String inheritedScope,
