@@ -25,27 +25,29 @@ import org.apache.maven.artifact.repository.ArtifactRepository;
 import org.apache.maven.artifact.resolver.ArtifactResolutionException;
 import org.apache.maven.artifact.versioning.DefaultArtifactVersion;
 import org.apache.maven.execution.BuildFailure;
+import org.apache.maven.execution.DefaultMavenExecutionRequest;
 import org.apache.maven.execution.MavenExecutionRequest;
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.execution.ReactorManager;
 import org.apache.maven.execution.RuntimeInformation;
 import org.apache.maven.lifecycle.LifecycleExecutionException;
 import org.apache.maven.lifecycle.LifecycleExecutor;
-import org.apache.maven.model.Profile;
+import org.apache.maven.monitor.event.DefaultEventDispatcher;
 import org.apache.maven.monitor.event.EventDispatcher;
 import org.apache.maven.monitor.event.MavenEvents;
 import org.apache.maven.profiles.ProfileManager;
 import org.apache.maven.profiles.activation.ProfileActivationException;
+import org.apache.maven.project.DefaultProjectBuilderConfiguration;
 import org.apache.maven.project.DuplicateProjectException;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.project.MavenProjectBuilder;
+import org.apache.maven.project.ProjectBuilderConfiguration;
 import org.apache.maven.project.ProjectBuildingException;
 import org.apache.maven.reactor.MavenExecutionException;
 import org.apache.maven.settings.Mirror;
 import org.apache.maven.settings.Proxy;
 import org.apache.maven.settings.Server;
 import org.apache.maven.settings.Settings;
-import org.apache.maven.settings.SettingsUtils;
 import org.apache.maven.usability.SystemWarnings;
 import org.apache.maven.usability.diagnostics.ErrorDiagnostics;
 import org.apache.maven.wagon.repository.RepositoryPermissions;
@@ -72,11 +74,12 @@ import java.util.Collections;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Properties;
 import java.util.TimeZone;
 
 /**
  * @author <a href="mailto:jason@maven.org">Jason van Zyl </a>
- * @version $Id: DefaultMaven.java 588043 2007-10-24 21:33:11Z carlos $
+ * @version $Id: DefaultMaven.java 642024 2008-03-27 23:30:59Z jdcasey $
  */
 public class DefaultMaven
     extends AbstractLogEnabled
@@ -286,7 +289,7 @@ public class DefaultMaven
         getLogger().info( "Scanning for projects..." );
 
         boolean foundProjects = true;
-        List projects = getProjects( request, globalProfileManager );
+        List projects = getProjects( request );
         if ( projects.isEmpty() )
         {
             projects.add( getSuperProject( request ) );
@@ -341,7 +344,7 @@ public class DefaultMaven
         MavenProject superProject;
         try
         {
-            superProject = projectBuilder.buildStandaloneSuperProject( request.getLocalRepository() );
+            superProject = projectBuilder.buildStandaloneSuperProject( request.getLocalRepository(), request.getGlobalProfileManager() );
 
         }
         catch ( ProjectBuildingException e )
@@ -351,7 +354,7 @@ public class DefaultMaven
         return superProject;
     }
 
-    private List getProjects( MavenExecutionRequest request, ProfileManager globalProfileManager )
+    private List getProjects( MavenExecutionRequest request )
         throws MavenExecutionException, BuildFailureException
     {
         List projects;
@@ -359,8 +362,7 @@ public class DefaultMaven
         {
             List files = getProjectFiles( request );
 
-            projects = collectProjects( files, request.getLocalRepository(), request.isRecursive(),
-                                        request.getSettings(), globalProfileManager, !request.isReactorActive() );
+            projects = collectProjects( files, request, !request.isReactorActive() );
 
         }
         catch ( IOException e )
@@ -436,11 +438,12 @@ public class DefaultMaven
         return fmt.format( new Date( time ) );
     }
 
-    private List collectProjects( List files, ArtifactRepository localRepository, boolean recursive, Settings settings,
-                                  ProfileManager globalProfileManager, boolean isRoot )
+    private List collectProjects( List files, MavenExecutionRequest request, boolean isRoot )
         throws ArtifactResolutionException, ProjectBuildingException, ProfileActivationException,
         MavenExecutionException, BuildFailureException
     {
+//        .getLocalRepository(), request.isRecursive(),
+//        request.getSettings(), request.getUserProperties(), requ, !request.isReactorActive()
         List projects = new ArrayList( files.size() );
 
         for ( Iterator iterator = files.iterator(); iterator.hasNext(); )
@@ -455,14 +458,14 @@ public class DefaultMaven
                 usingReleasePom = true;
             }
 
-            MavenProject project = getProject( file, localRepository, settings, globalProfileManager );
+            MavenProject project = getProject( file, request );
 
             if ( isRoot )
             {
                 project.setExecutionRoot( true );
             }
 
-            if ( project.getPrerequisites() != null && project.getPrerequisites().getMaven() != null )
+            if ( ( project.getPrerequisites() != null ) && ( project.getPrerequisites().getMaven() != null ) )
             {
                 DefaultArtifactVersion version = new DefaultArtifactVersion( project.getPrerequisites().getMaven() );
                 if ( runtimeInformation.getApplicationVersion().compareTo( version ) < 0 )
@@ -472,7 +475,7 @@ public class DefaultMaven
                 }
             }
 
-            if ( project.getModules() != null && !project.getModules().isEmpty() && recursive )
+            if ( ( project.getModules() != null ) && !project.getModules().isEmpty() && request.isRecursive() )
             {
                 // TODO: Really should fail if it was not? What if it is aggregating - eg "ear"?
                 project.setPackaging( "pom" );
@@ -493,15 +496,18 @@ public class DefaultMaven
                         continue;
                     }
 
-                    File moduleFile;
+                    File moduleFile = new File( basedir, name );
 
-                    if ( usingReleasePom )
+                    if ( moduleFile.exists() && moduleFile.isDirectory() )
                     {
-                        moduleFile = new File( basedir, name + "/" + Maven.RELEASE_POMv4 );
-                    }
-                    else
-                    {
-                        moduleFile = new File( basedir, name + "/" + Maven.POMv4 );
+                        if ( usingReleasePom )
+                        {
+                            moduleFile = new File( basedir, name + "/" + Maven.RELEASE_POMv4 );
+                        }
+                        else
+                        {
+                            moduleFile = new File( basedir, name + "/" + Maven.POMv4 );
+                        }
                     }
 
                     if ( Os.isFamily( "windows" ) )
@@ -517,12 +523,16 @@ public class DefaultMaven
                             throw new MavenExecutionException( "Unable to canonicalize file name " + moduleFile, e );
                         }
                     }
+                    else
+                    {
+                        moduleFile = new File( moduleFile.toURI().normalize() );
+                    }
 
                     moduleFiles.add( moduleFile );
                 }
 
                 List collectedProjects =
-                    collectProjects( moduleFiles, localRepository, recursive, settings, globalProfileManager, false );
+                    collectProjects( moduleFiles, request, false );
                 projects.addAll( collectedProjects );
                 project.setCollectedProjects( collectedProjects );
             }
@@ -532,8 +542,28 @@ public class DefaultMaven
         return projects;
     }
 
+    /**
+     * @deprecated Use {@link DefaultMaven#getProject(File, MavenExecutionRequest)} instead.
+     */
     public MavenProject getProject( File pom, ArtifactRepository localRepository, Settings settings,
-                                    ProfileManager globalProfileManager )
+                                    Properties userProperties, ProfileManager globalProfileManager )
+        throws ProjectBuildingException, ArtifactResolutionException, ProfileActivationException
+    {
+        MavenExecutionRequest request = new DefaultMavenExecutionRequest(
+                                                                      localRepository,
+                                                                      settings,
+                                                                      new DefaultEventDispatcher(),
+                                                                      Collections.EMPTY_LIST,
+                                                                      pom.getParentFile()
+                                                                         .getAbsolutePath(),
+                                                                      globalProfileManager,
+                                                                      globalProfileManager.getRequestProperties(),
+                                                                      new Properties(), false );
+
+        return getProject( pom, request );
+    }
+
+    public MavenProject getProject( File pom, MavenExecutionRequest request )
         throws ProjectBuildingException, ArtifactResolutionException, ProfileActivationException
     {
         if ( pom.exists() )
@@ -545,7 +575,12 @@ public class DefaultMaven
             }
         }
 
-        return projectBuilder.build( pom, localRepository, globalProfileManager );
+        ProjectBuilderConfiguration config = new DefaultProjectBuilderConfiguration();
+        config.setLocalRepository( request.getLocalRepository() )
+              .setGlobalProfileManager( request.getGlobalProfileManager() )
+              .setUserProperties( request.getUserProperties() );
+
+        return projectBuilder.build( pom, config );
     }
 
     // ----------------------------------------------------------------------
