@@ -19,14 +19,6 @@ package org.apache.maven.cli;
  * under the License.
  */
 
-import java.io.File;
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.List;
-import java.util.Properties;
-import java.util.StringTokenizer;
-
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
 import org.apache.commons.cli.GnuParser;
@@ -63,11 +55,22 @@ import org.codehaus.plexus.embed.Embedder;
 import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.logging.LoggerManager;
 import org.codehaus.plexus.util.Os;
+import org.codehaus.plexus.util.cli.CommandLineUtils;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Properties;
+import java.util.StringTokenizer;
+import java.util.Map.Entry;
+
 /**
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
- * @version $Id: MavenCli.java 587686 2007-10-23 23:01:40Z carlos $
+ * @version $Id: MavenCli.java 642024 2008-03-27 23:30:59Z jdcasey $
  * @noinspection UseOfSystemOutOrSystemErr,ACCESS_STATIC_VIA_INSTANCE
  */
 public class MavenCli
@@ -130,7 +133,7 @@ public class MavenCli
         {
             System.out.println( "+ Error stacktraces are turned on." );
         }
-  
+
         // ----------------------------------------------------------------------
         // Process particular command line options
         // ----------------------------------------------------------------------
@@ -177,7 +180,9 @@ public class MavenCli
         // are constructed.
         // ----------------------------------------------------------------------
 
-        Properties executionProperties = getExecutionProperties( commandLine );
+        Properties executionProperties = new Properties();
+        Properties userProperties = new Properties();
+        populateProperties( commandLine, executionProperties, userProperties );
 
         Settings settings;
 
@@ -221,14 +226,14 @@ public class MavenCli
                 // TODO:Additionally, we can't change the mojo level because the component key includes the version and it isn't known ahead of time. This seems worth changing.
             }
 
-            ProfileManager profileManager = new DefaultProfileManager( embedder.getContainer(),System.getProperties() );
+            ProfileManager profileManager = new DefaultProfileManager( embedder.getContainer(), executionProperties );
 
             if ( commandLine.hasOption( CLIManager.ACTIVATE_PROFILES ) )
             {
                 String profilesLine = commandLine.getOptionValue( CLIManager.ACTIVATE_PROFILES );
 
                 StringTokenizer profileTokens = new StringTokenizer( profilesLine, "," );
-                
+
                 while ( profileTokens.hasMoreTokens() )
                 {
                     String profileAction = profileTokens.nextToken().trim();
@@ -250,7 +255,7 @@ public class MavenCli
             }
 
             request = createRequest( commandLine, settings, eventDispatcher, loggerManager, profileManager,
-                                     executionProperties, showErrors );
+                                     executionProperties, userProperties, showErrors );
 
             setProjectFileOptions( commandLine, request );
 
@@ -401,7 +406,7 @@ public class MavenCli
     private static MavenExecutionRequest createRequest( CommandLine commandLine, Settings settings,
                                                         EventDispatcher eventDispatcher, LoggerManager loggerManager,
                                                         ProfileManager profileManager, Properties executionProperties,
-                                                        boolean showErrors )
+                                                        Properties userProperties, boolean showErrors )
         throws ComponentLookupException
     {
         MavenExecutionRequest request;
@@ -412,7 +417,7 @@ public class MavenCli
 
         request = new DefaultMavenExecutionRequest( localRepository, settings, eventDispatcher,
                                                     commandLine.getArgList(), userDir.getPath(), profileManager,
-                                                    executionProperties, showErrors );
+                                                    executionProperties, userProperties, showErrors );
 
         // TODO [BP]: do we set one per mojo? where to do it?
         Logger logger = loggerManager.getLoggerForComponent( Mojo.ROLE );
@@ -533,23 +538,31 @@ public class MavenCli
             Properties properties = new Properties();
             resourceAsStream = MavenCli.class.getClassLoader().getResourceAsStream(
                 "META-INF/maven/org.apache.maven/maven-core/pom.properties" );
-            properties.load( resourceAsStream );
 
-            if( properties.getProperty( "builtOn" ) != null )
+            if ( resourceAsStream != null )
             {
-                System.out.println( "Maven version: " + properties.getProperty( "version", "unknown" )
-                    + " built on " + properties.getProperty( "builtOn" ) );
+                properties.load( resourceAsStream );
+
+                if( properties.getProperty( "builtOn" ) != null )
+                {
+                    System.out.println( "Maven version: " + properties.getProperty( "version", "unknown" )
+                        + " built on " + properties.getProperty( "builtOn" ) );
+                }
+                else
+                {
+                    System.out.println( "Maven version: " + properties.getProperty( "version", "unknown" ) );
+                }
             }
             else
             {
-                System.out.println( "Maven version: " + properties.getProperty( "version", "unknown" ) );
+                System.out.println( "Maven version: unknown" );
             }
-            
+
             System.out.println( "Java version: " + System.getProperty( "java.version", "<unknown java version>" ) );
 
             System.out.println( "OS name: \"" + Os.OS_NAME + "\" version: \"" + Os.OS_VERSION +
                                 "\" arch: \"" + Os.OS_ARCH + "\" Family: \"" + Os.OS_FAMILY + "\"" );
-                                           
+
         }
         catch ( IOException e )
         {
@@ -561,9 +574,24 @@ public class MavenCli
     // System properties handling
     // ----------------------------------------------------------------------
 
-    private static Properties getExecutionProperties( CommandLine commandLine )
+    static void populateProperties( CommandLine commandLine, Properties executionProperties, Properties userProperties )
     {
-        Properties executionProperties = new Properties();
+        // add the env vars to the property set, with the "env." prefix
+        // XXX support for env vars should probably be removed from the ModelInterpolator
+        try
+        {
+            Properties envVars = CommandLineUtils.getSystemEnvVars();
+            Iterator i = envVars.entrySet().iterator();
+            while ( i.hasNext() )
+            {
+                Entry e = (Entry) i.next();
+                executionProperties.setProperty( "env." + e.getKey().toString(), e.getValue().toString() );
+            }
+        }
+        catch ( IOException e )
+        {
+            System.err.println( "Error getting environment vars for profile activation: " + e );
+        }
 
         // ----------------------------------------------------------------------
         // Options that are set on the command line become system properties
@@ -575,18 +603,21 @@ public class MavenCli
         {
             String[] defStrs = commandLine.getOptionValues( CLIManager.SET_SYSTEM_PROPERTY );
 
-            for ( int i = 0; i < defStrs.length; ++i )
+            if ( defStrs != null )
             {
-                setCliProperty( defStrs[i], executionProperties );
+                for ( int i = 0; i < defStrs.length; ++i )
+                {
+                    setCliProperty( defStrs[i], userProperties );
+                }
             }
+
+            executionProperties.putAll( userProperties );
         }
 
         executionProperties.putAll( System.getProperties() );
-
-        return executionProperties;
     }
 
-    private static void setCliProperty( String property, Properties executionProperties )
+    private static void setCliProperty( String property, Properties requestProperties )
     {
         String name;
 
@@ -607,7 +638,7 @@ public class MavenCli
             value = property.substring( i + 1 ).trim();
         }
 
-        executionProperties.setProperty( name, value );
+        requestProperties.setProperty( name, value );
 
         // ----------------------------------------------------------------------
         // I'm leaving the setting of system properties here as not to break
