@@ -1,3 +1,4 @@
+
 package org.apache.maven.plugin;
 
 /*
@@ -19,6 +20,20 @@ package org.apache.maven.plugin;
  * under the License.
  */
 
+import java.io.File;
+import java.net.URL;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
 import org.apache.maven.MavenArtifactFilterManager;
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.factory.ArtifactFactory;
@@ -59,6 +74,8 @@ import org.apache.maven.project.artifact.MavenMetadataSource;
 import org.apache.maven.project.path.PathTranslator;
 import org.apache.maven.reporting.MavenReport;
 import org.apache.maven.settings.Settings;
+import org.codehaus.classworlds.ClassRealm;
+import org.codehaus.classworlds.NoSuchRealmException;
 import org.codehaus.plexus.PlexusConstants;
 import org.codehaus.plexus.PlexusContainer;
 import org.codehaus.plexus.PlexusContainerException;
@@ -78,19 +95,6 @@ import org.codehaus.plexus.personality.plexus.lifecycle.phase.Contextualizable;
 import org.codehaus.plexus.personality.plexus.lifecycle.phase.Initializable;
 import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
-import org.codehaus.classworlds.NoSuchRealmException;
-
-import java.io.File;
-import java.net.URL;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
 
 public class DefaultPluginManager
     extends AbstractLogEnabled
@@ -460,6 +464,45 @@ public class DefaultPluginManager
 
             throw e;
         }
+        catch ( LinkageError e )
+        {
+            if ( getLogger().isFatalErrorEnabled() )
+            {
+                getLogger().fatalError(
+                                        plugin.getClass().getName() + "#execute() caused a linkage error ("
+                                            + e.getClass().getName() + ") and may be out-of-date. Check the realms:" );
+
+                ClassRealm pluginRealm = mojoDescriptor.getPluginDescriptor().getClassRealm();
+                StringBuffer sb = new StringBuffer();
+                sb.append( "Plugin realm = " + pluginRealm.getId() ).append( '\n' );
+                for ( int i = 0; i < pluginRealm.getConstituents().length; i++ )
+                {
+                    sb.append( "urls[" + i + "] = " + pluginRealm.getConstituents()[i] );
+                    if ( i != ( pluginRealm.getConstituents().length - 1 ) )
+                    {
+                        sb.append( '\n' );
+                    }
+                }
+                getLogger().fatalError( sb.toString() );
+
+                ClassRealm containerRealm = container.getContainerRealm();
+                sb = new StringBuffer();
+                sb.append( "Container realm = " + containerRealm.getId() ).append( '\n' );
+                for ( int i = 0; i < containerRealm.getConstituents().length; i++ )
+                {
+                    sb.append( "urls[" + i + "] = " + containerRealm.getConstituents()[i] );
+                    if ( i != ( containerRealm.getConstituents().length - 1 ) )
+                    {
+                        sb.append( '\n' );
+                    }
+                }
+                getLogger().fatalError( sb.toString() );
+            }
+
+            session.getEventDispatcher().dispatchError( event, goalExecId, e );
+
+            throw e;
+        }
         finally
         {
 
@@ -648,15 +691,84 @@ public class DefaultPluginManager
 
             checkPlexusUtils( resolutionGroup, artifactFactory );
 
-            Set dependencies = new HashSet( resolutionGroup.getArtifacts() );
-            dependencies.addAll( pluginDescriptor.getIntroducedDependencyArtifacts() );
+            // [jdcasey; 20-March-2008]:
+            // This is meant to eliminate the introduction of duplicated artifacts.
+            // Since much of the reasoning for reversing the order of introduction of
+            // plugin dependencies rests on the notion that we need to be able to
+            // introduce upgraded versions of plugin dependencies on a case-by-case
+            // basis, we need to remove the original version prior to artifact
+            // resolution. This is consistent with recent thinking on duplicated
+            // dependency specifications within a POM, where that case should
+            // throw a model validation exception.
+            //
+            // Here, we just want to remove any chance that the ArtifactCollector
+            // could make a bad choice, and use the old version in spite of our
+            // explicit preference otherwise.
+
+            // First, we're going to accumulate plugin dependencies in an ordered map,
+            // keyed by dependencyConflictId (the ordered map is meant to preserve relative
+            // ordering of the dependencies that do make the cut).
+            Map dependencyMap = new LinkedHashMap();
+
+            // Next, we need to accumulate all dependencies in a List, to make it
+            // simpler to iterate through them all and add them to the map.
+            List all = new ArrayList();
+
+            // plugin-level dependencies from the consuming POM override dependencies
+            // from the plugin's own POM.
+            all.addAll( pluginDescriptor.getIntroducedDependencyArtifacts() );
+
+            // add in the deps from the plugin POM now.
+            all.addAll( resolutionGroup.getArtifacts() );
+
+            for ( Iterator it = all.iterator(); it.hasNext(); )
+            {
+                Artifact artifact = (Artifact) it.next();
+                String conflictId = artifact.getDependencyConflictId();
+
+                // if the map already contains this dependencyConflictId, it constitutes an
+                // overridden dependency. Don't use the old one (we know it's old from the
+                // order in which dependencies were added to this list).
+                if ( !dependencyMap.containsKey( conflictId ) )
+                {
+                    dependencyMap.put( conflictId, artifact );
+                }
+            }
+
+            // Create an ordered set of dependencies from the ordered map we used above, to feed into the resolver.
+            Set dependencies = new LinkedHashSet( dependencyMap.values() );
+
+            if ( getLogger().isDebugEnabled() )
+            {
+                // list all dependencies to be used by this plugin (first-level deps, not transitive ones).
+                getLogger().debug( "Plugin dependencies for:\n\n" + pluginDescriptor.getId()
+                                   + "\n\nare:\n\n"
+                                   + StringUtils.join( dependencies.iterator(), "\n" ) + "\n\n" );
+            }
 
             List repositories = new ArrayList();
             repositories.addAll( resolutionGroup.getResolutionRepositories() );
             repositories.addAll( project.getRemoteArtifactRepositories() );
 
+            /* get plugin managed versions */
+            Map pluginManagedDependencies = new HashMap();
+            try
+            {
+                MavenProject pluginProject =
+                    mavenProjectBuilder.buildFromRepository( pluginArtifact, project.getRemoteArtifactRepositories(),
+                                                             localRepository );
+                if ( pluginProject != null )
+                {
+                    pluginManagedDependencies = pluginProject.getManagedVersionMap();
+                }
+            }
+            catch ( ProjectBuildingException e )
+            {
+                // this can't happen, it would have blowed up at artifactMetadataSource.retrieve()
+            }
+
             ArtifactResolutionResult result = artifactResolver.resolveTransitively( dependencies, pluginArtifact,
-                                                                                    Collections.EMPTY_MAP,
+                                                                                    pluginManagedDependencies,
                                                                                     localRepository, repositories,
                                                                                     artifactMetadataSource,
                                                                                     artifactFilter );
@@ -689,8 +801,27 @@ public class DefaultPluginManager
 
             unresolved.removeAll( resolved );
 
+            if ( getLogger().isDebugEnabled() )
+            {
+                // list all artifacts that were filtered out during the resolution process.
+                // these are already present in the core container.
+                getLogger().debug( " The following artifacts were filtered out for plugin: "
+                                   + pluginDescriptor.getId()
+                                   + " because they're already in the core of Maven:\n\n"
+                                   + StringUtils.join( unresolved.iterator(), "\n" )
+                                   + "\n\nThese will use the artifact files already in the core ClassRealm instead, to allow them to be included in PluginDescriptor.getArtifacts().\n\n" );
+            }
+
+            // Grab a file for all filtered artifacts, even if it means resolving them. This
+            // is necessary in order to present a full complement of a plugin's transitive
+            // dependencies to anyone who calls PluginDescriptor.getArtifacts().
             resolveCoreArtifacts( unresolved, localRepository, resolutionGroup.getResolutionRepositories() );
 
+            // Re-join resolved and filtered-but-now-resolved artifacts.
+            // NOTE: The process of filtering then re-adding some artifacts will
+            // result in different ordering within the PluginDescriptor.getArtifacts()
+            // List than should have happened if none had been filtered. All filtered
+            // artifacts will be listed last...
             List allResolved = new ArrayList( resolved.size() + unresolved.size() );
 
             allResolved.addAll( resolved );
@@ -1163,6 +1294,44 @@ public class DefaultPluginManager
                                                     "Unable to retrieve component configurator for plugin configuration",
                                                     e );
         }
+        catch ( LinkageError e )
+        {
+            if ( getLogger().isFatalErrorEnabled() )
+            {
+                getLogger().fatalError(
+                                        configurator.getClass().getName() + "#configureComponent(...) caused a linkage error ("
+                                            + e.getClass().getName() + ") and may be out-of-date. Check the realms:" );
+
+                ClassRealm pluginRealm = mojoDescriptor.getPluginDescriptor().getClassRealm();
+                StringBuffer sb = new StringBuffer();
+                sb.append( "Plugin realm = " + pluginRealm.getId() ).append( '\n' );
+                for ( int i = 0; i < pluginRealm.getConstituents().length; i++ )
+                {
+                    sb.append( "urls[" + i + "] = " + pluginRealm.getConstituents()[i] );
+                    if ( i != ( pluginRealm.getConstituents().length - 1 ) )
+                    {
+                        sb.append( '\n' );
+                    }
+                }
+                getLogger().fatalError( sb.toString() );
+
+                ClassRealm containerRealm = container.getContainerRealm();
+                sb = new StringBuffer();
+                sb.append( "Container realm = " + containerRealm.getId() ).append( '\n' );
+                for ( int i = 0; i < containerRealm.getConstituents().length; i++ )
+                {
+                    sb.append( "urls[" + i + "] = " + containerRealm.getConstituents()[i] );
+                    if ( i != ( containerRealm.getConstituents().length - 1 ) )
+                    {
+                        sb.append( '\n' );
+                    }
+                }
+                getLogger().fatalError( sb.toString() );
+            }
+
+            throw new PluginConfigurationException( mojoDescriptor.getPluginDescriptor(),
+                                                    e.getClass().getName() + ": " + e.getMessage(), e );
+        }
         finally
         {
             if ( configurator != null )
