@@ -35,8 +35,8 @@ import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
 import org.apache.maven.artifact.resolver.ArtifactResolver;
 import org.apache.maven.artifact.resolver.filter.ExcludesArtifactFilter;
 import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
-import org.apache.maven.artifact.versioning.VersionRange;
 import org.apache.maven.artifact.versioning.ManagedVersionMap;
+import org.apache.maven.artifact.versioning.VersionRange;
 import org.apache.maven.model.Build;
 import org.apache.maven.model.Dependency;
 import org.apache.maven.model.DependencyManagement;
@@ -75,8 +75,8 @@ import org.codehaus.plexus.logging.AbstractLogEnabled;
 import org.codehaus.plexus.personality.plexus.lifecycle.phase.Contextualizable;
 import org.codehaus.plexus.personality.plexus.lifecycle.phase.Initializable;
 import org.codehaus.plexus.util.IOUtil;
-import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.ReaderFactory;
+import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 
 import java.io.File;
@@ -95,7 +95,9 @@ import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Set;
+import java.util.TreeMap;
 
 /*:apt
 
@@ -132,7 +134,7 @@ Notes
 */
 
 /**
- * @version $Id: DefaultMavenProjectBuilder.java 585268 2007-10-16 21:43:07Z hboutemy $
+ * @version $Id: DefaultMavenProjectBuilder.java 644411 2008-04-03 17:15:48Z jdcasey $
  */
 public class DefaultMavenProjectBuilder
     extends AbstractLogEnabled
@@ -189,12 +191,28 @@ public class DefaultMavenProjectBuilder
     // MavenProjectBuilder Implementation
     // ----------------------------------------------------------------------
 
+    public MavenProject build( File pom,
+                               ProjectBuilderConfiguration config )
+        throws ProjectBuildingException
+    {
+        return buildFromSourceFileInternal( pom, config, true );
+    }
+
+    public MavenProject build( File pom,
+                               ProjectBuilderConfiguration config,
+                               boolean checkDistributionManagementStatus )
+        throws ProjectBuildingException
+    {
+        return buildFromSourceFileInternal( pom, config, checkDistributionManagementStatus );
+    }
+
     public MavenProject build( File projectDescriptor,
                                ArtifactRepository localRepository,
                                ProfileManager profileManager )
         throws ProjectBuildingException
     {
-        return buildFromSourceFileInternal( projectDescriptor, localRepository, profileManager, true );
+        ProjectBuilderConfiguration config = new DefaultProjectBuilderConfiguration().setLocalRepository( localRepository ).setGlobalProfileManager( profileManager );
+        return buildFromSourceFileInternal( projectDescriptor, config, true );
     }
 
     public MavenProject build( File projectDescriptor,
@@ -203,8 +221,8 @@ public class DefaultMavenProjectBuilder
                                boolean checkDistributionManagementStatus )
         throws ProjectBuildingException
     {
-        return buildFromSourceFileInternal( projectDescriptor, localRepository, profileManager,
-                                            checkDistributionManagementStatus );
+        ProjectBuilderConfiguration config = new DefaultProjectBuilderConfiguration().setLocalRepository( localRepository ).setGlobalProfileManager( profileManager );
+        return buildFromSourceFileInternal( projectDescriptor, config, checkDistributionManagementStatus );
     }
 
     // jvz:note
@@ -230,7 +248,9 @@ public class DefaultMavenProjectBuilder
 
         Model model = findModelFromRepository( artifact, remoteArtifactRepositories, localRepository, allowStubModel );
 
-        return buildInternal( "Artifact [" + artifact + "]", model, localRepository, remoteArtifactRepositories, null,
+        ProjectBuilderConfiguration config = new DefaultProjectBuilderConfiguration().setLocalRepository( localRepository );
+
+        return buildInternal( "Artifact [" + artifact + "]", model, config, remoteArtifactRepositories,
                               null, false );
     }
 
@@ -248,12 +268,21 @@ public class DefaultMavenProjectBuilder
     {
         //TODO mkleint - use the (Container, Properties) constructor to make system properties embeddable
         ProfileManager profileManager = new DefaultProfileManager( container );
-        return buildStandaloneSuperProject( localRepository, profileManager );
+
+        return buildStandaloneSuperProject( new DefaultProjectBuilderConfiguration().setLocalRepository( localRepository )
+                                                                                    .setGlobalProfileManager( profileManager ) );
     }
 
     public MavenProject buildStandaloneSuperProject( ArtifactRepository localRepository,
                                                      ProfileManager profileManager )
         throws ProjectBuildingException
+    {
+        return buildStandaloneSuperProject( new DefaultProjectBuilderConfiguration().setLocalRepository( localRepository )
+                                                                                    .setGlobalProfileManager( profileManager ) );
+    }
+
+    public MavenProject buildStandaloneSuperProject( ProjectBuilderConfiguration config )
+        throws ProjectBuildingException
     {
         Model superModel = getSuperModel();
 
@@ -266,6 +295,8 @@ public class DefaultMavenProjectBuilder
 
         List activeProfiles;
 
+        ProfileManager profileManager = config.getGlobalProfileManager();
+
         if ( profileManager == null )
         {
             profileManager = new DefaultProfileManager( container );
@@ -288,7 +319,7 @@ public class DefaultMavenProjectBuilder
 
         try
         {
-            project = processProjectLogic( "<Super-POM>", project, null, null, true );
+            project = processProjectLogic( "<Super-POM>", project, config, null, null, true, true );
 
             project.setExecutionRoot( true );
 
@@ -400,8 +431,8 @@ public class DefaultMavenProjectBuilder
     {
         Map map = null;
         List deps;
-        if ( dependencyManagement != null && ( deps = dependencyManagement.getDependencies() ) != null &&
-            deps.size() > 0 )
+        if ( ( dependencyManagement != null ) && ( ( deps = dependencyManagement.getDependencies() ) != null ) &&
+            ( deps.size() > 0 ) )
         {
             map = new ManagedVersionMap( map );
 
@@ -430,7 +461,7 @@ public class DefaultMavenProjectBuilder
                     // If the dependencyManagement section listed exclusions,
                     // add them to the managed artifacts here so that transitive
                     // dependencies will be excluded if necessary.
-                    if ( null != d.getExclusions() && !d.getExclusions().isEmpty() )
+                    if ( ( null != d.getExclusions() ) && !d.getExclusions().isEmpty() )
                     {
                         List exclusions = new ArrayList();
 
@@ -466,21 +497,20 @@ public class DefaultMavenProjectBuilder
     }
 
     private MavenProject buildFromSourceFileInternal( File projectDescriptor,
-                                                      ArtifactRepository localRepository,
-                                                      ProfileManager profileManager,
+                                                      ProjectBuilderConfiguration config,
                                                       boolean checkDistributionManagementStatus )
         throws ProjectBuildingException
     {
         Model model = readModel( "unknown", projectDescriptor, true );
 
-        MavenProject project = buildInternal( projectDescriptor.getAbsolutePath(), model, localRepository,
+        MavenProject project = buildInternal( projectDescriptor.getAbsolutePath(), model, config,
                                               buildArtifactRepositories( getSuperModel() ), projectDescriptor,
-                                              profileManager, true );
+                                              true );
 
         if ( checkDistributionManagementStatus )
         {
-            if ( project.getDistributionManagement() != null &&
-                project.getDistributionManagement().getStatus() != null )
+            if ( ( project.getDistributionManagement() != null ) &&
+                ( project.getDistributionManagement().getStatus() != null ) )
             {
                 String projectId = safeVersionlessKey( project.getGroupId(), project.getArtifactId() );
 
@@ -498,6 +528,10 @@ public class DefaultMavenProjectBuilder
                                            boolean allowStubModel )
         throws ProjectBuildingException
     {
+        String projectId = safeVersionlessKey( artifact.getGroupId(), artifact.getArtifactId() );
+
+        normalizeToArtifactRepositories( remoteArtifactRepositories, projectId );
+
         Artifact projectArtifact;
 
         // if the artifact is not a POM, we need to construct a POM artifact based on the artifact parameter given.
@@ -517,8 +551,6 @@ public class DefaultMavenProjectBuilder
 
         Model model;
 
-        String projectId = ArtifactUtils.versionlessKey( projectArtifact );
-
         try
         {
             artifactResolver.resolve( projectArtifact, remoteArtifactRepositories, localRepository );
@@ -576,6 +608,52 @@ public class DefaultMavenProjectBuilder
         return model;
     }
 
+    private List normalizeToArtifactRepositories( List remoteArtifactRepositories,
+                                                  String projectId )
+        throws ProjectBuildingException
+    {
+        List normalized = new ArrayList( remoteArtifactRepositories.size() );
+
+        boolean normalizationNeeded = false;
+        for ( Iterator it = remoteArtifactRepositories.iterator(); it.hasNext(); )
+        {
+            Object item = it.next();
+
+            if ( item instanceof ArtifactRepository )
+            {
+                normalized.add( item );
+            }
+            else if ( item instanceof Repository )
+            {
+                Repository repo = (Repository) item;
+                try
+                {
+                    item = ProjectUtils.buildArtifactRepository( repo, artifactRepositoryFactory, container );
+
+                    normalized.add( item );
+                    normalizationNeeded = true;
+                }
+                catch ( InvalidRepositoryException e )
+                {
+                    throw new ProjectBuildingException( projectId, "Error building artifact repository for id: " + repo.getId(), e );
+                }
+            }
+            else
+            {
+                throw new ProjectBuildingException( projectId, "Error building artifact repository from non-repository information item: " + item );
+            }
+        }
+
+        if ( normalizationNeeded )
+        {
+            return normalized;
+        }
+        else
+        {
+            return remoteArtifactRepositories;
+        }
+    }
+
     private void checkStatusAndUpdate( Artifact projectArtifact,
                                        ArtifactStatus status,
                                        File file,
@@ -584,7 +662,7 @@ public class DefaultMavenProjectBuilder
         throws ArtifactNotFoundException
     {
         // TODO: configurable actions dependant on status
-        if ( !projectArtifact.isSnapshot() && status.compareTo( ArtifactStatus.DEPLOYED ) < 0 )
+        if ( !projectArtifact.isSnapshot() && ( status.compareTo( ArtifactStatus.DEPLOYED ) < 0 ) )
         {
             // use default policy (enabled, daily update, warn on bad checksum)
             ArtifactRepositoryPolicy policy = new ArtifactRepositoryPolicy();
@@ -648,10 +726,9 @@ public class DefaultMavenProjectBuilder
     // is a real file and sometimes null which makes things confusing.
     private MavenProject buildInternal( String pomLocation,
                                         Model model,
-                                        ArtifactRepository localRepository,
+                                        ProjectBuilderConfiguration config,
                                         List parentSearchRepositories,
                                         File projectDescriptor,
-                                        ProfileManager externalProfileManager,
                                         boolean strict )
         throws ProjectBuildingException
     {
@@ -664,14 +741,13 @@ public class DefaultMavenProjectBuilder
 
         Model superModel = getSuperModel();
 
-        //TODO mkleint - use the (Container, Properties) constructor to make system properties embeddable
-        // shall the ProfileManager intefrace expose the properties?
-
+        ProfileManager externalProfileManager = config.getGlobalProfileManager();
         ProfileManager superProjectProfileManager;
-        if ( externalProfileManager instanceof DefaultProfileManager )
+        if ( externalProfileManager != null )
         {
-            superProjectProfileManager = new DefaultProfileManager( container,
-                                                                    ( (DefaultProfileManager) externalProfileManager ).getSystemProperties() );
+            superProjectProfileManager = new DefaultProfileManager(
+                                                                    container,
+                                                                    externalProfileManager.getRequestProperties() );
         }
         else
         {
@@ -742,8 +818,8 @@ public class DefaultMavenProjectBuilder
         MavenProject project = null;
         try
         {
-            project = assembleLineage( model, lineage, localRepository, projectDir, parentSearchRepositories,
-                                       aggregatedRemoteWagonRepositories, externalProfileManager, strict );
+            project = assembleLineage( model, lineage, config, projectDir, parentSearchRepositories,
+                                       aggregatedRemoteWagonRepositories, strict );
         }
         catch ( InvalidRepositoryException e )
         {
@@ -794,9 +870,12 @@ public class DefaultMavenProjectBuilder
             }
         }
 
+        // merge any duplicated plugin definitions together, using the first appearance as the dominant one.
+        ModelUtils.mergeDuplicatePluginDefinitions( project.getModel().getBuild() );
+
         try
         {
-            project = processProjectLogic( pomLocation, project, externalProfileManager, projectDir, strict );
+            project = processProjectLogic( pomLocation, project, config, projectDir, repositories, strict, false );
         }
         catch ( ModelInterpolationException e )
         {
@@ -808,14 +887,15 @@ public class DefaultMavenProjectBuilder
         }
 
         processedProjectCache.put(
-            createCacheKey( project.getGroupId(), project.getArtifactId(), project.getVersion() ), project );
+                                  createCacheKey( project.getGroupId(), project.getArtifactId(), project.getVersion() ), project );
 
-        // jvz:note
+          // jvz:note
         // this only happens if we are building from a source file
         if ( projectDescriptor != null )
         {
             // Only translate the base directory for files in the source tree
-            pathTranslator.alignToBaseDirectory( project.getModel(), projectDescriptor.getParentFile() );
+            pathTranslator.alignToBaseDirectory( project.getModel(),
+                                                 projectDir );
 
             Build build = project.getBuild();
 
@@ -829,24 +909,9 @@ public class DefaultMavenProjectBuilder
             project.setFile( projectDescriptor );
         }
 
-        MavenProject rawParent = project.getParent();
-
-        if ( rawParent != null )
-        {
-            String cacheKey =
-                createCacheKey( rawParent.getGroupId(), rawParent.getArtifactId(), rawParent.getVersion() );
-
-            MavenProject processedParent = (MavenProject) processedProjectCache.get( cacheKey );
-
-            // yeah, this null check might be a bit paranoid, but better safe than sorry...
-            if ( processedParent != null )
-            {
-                project.setParent( processedParent );
-            }
-        }
-
-        project.setManagedVersionMap(
-            createManagedVersionMap( projectId, project.getDependencyManagement(), project.getParent() ) );
+        project.setManagedVersionMap( createManagedVersionMap( projectId,
+                                                               project.getDependencyManagement(),
+                                                               project.getParent() ) );
 
         return project;
     }
@@ -896,9 +961,11 @@ public class DefaultMavenProjectBuilder
      */
     private MavenProject processProjectLogic( String pomLocation,
                                               MavenProject project,
-                                              ProfileManager profileMgr,
+                                              ProjectBuilderConfiguration config,
                                               File projectDir,
-                                              boolean strict )
+                                              List remoteRepositories,
+                                              boolean strict,
+                                              boolean isSuperPom )
         throws ProjectBuildingException, ModelInterpolationException, InvalidRepositoryException
     {
         Model model = project.getModel();
@@ -910,32 +977,59 @@ public class DefaultMavenProjectBuilder
             activeProfiles = new ArrayList();
         }
 
+        ProfileManager profileMgr = config == null ? null : config.getGlobalProfileManager();
+
         List injectedProfiles = injectActiveProfiles( profileMgr, model );
 
         activeProfiles.addAll( injectedProfiles );
 
-        // TODO: Clean this up...we're using this to 'jump' the interpolation step for model properties not expressed in XML.
-        //  [BP] - Can this above comment be explained?
         // We don't need all the project methods that are added over those in the model, but we do need basedir
-        Map context = new HashMap( System.getProperties() );
+        Map context = new HashMap();
+
+        Build build = model.getBuild();
 
         if ( projectDir != null )
         {
             context.put( "basedir", projectDir.getAbsolutePath() );
+
+            // MNG-1927, MNG-2124, MNG-3355:
+            // If the build section is present and the project directory is non-null, we should make
+            // sure interpolation of the directories below uses translated paths.
+            // Afterward, we'll double back and translate any paths that weren't covered during interpolation via the
+            // code below...
+            context.put( "build.directory", pathTranslator.alignToBaseDirectory( build.getDirectory(), projectDir ) );
+            context.put( "build.outputDirectory", pathTranslator.alignToBaseDirectory( build.getOutputDirectory(), projectDir ) );
+            context.put( "build.testOutputDirectory", pathTranslator.alignToBaseDirectory( build.getTestOutputDirectory(), projectDir ) );
+            context.put( "build.sourceDirectory", pathTranslator.alignToBaseDirectory( build.getSourceDirectory(), projectDir ) );
+            context.put( "build.testSourceDirectory", pathTranslator.alignToBaseDirectory( build.getTestSourceDirectory(), projectDir ) );
         }
 
-        // TODO: this is a hack to ensure MNG-2124 can be satisfied without triggering MNG-1927
-        //  MNG-1927 relies on the false assumption that ${project.build.*} evaluates to null, which occurs before
-        //  MNG-2124 is fixed. The null value would leave it uninterpolated, to be handled after path translation.
-        //  Until these steps are correctly sequenced, we guarantee these fields remain uninterpolated.
-        context.put( "build.directory", null );
-        context.put( "build.outputDirectory", null );
-        context.put( "build.testOutputDirectory", null );
-        context.put( "build.sourceDirectory", null );
-        context.put( "build.testSourceDirectory", null );
+        if ( !isSuperPom )
+        {
+            Properties userProps = config.getUserProperties();
+            if ( userProps != null )
+            {
+                context.putAll( userProps );
+            }
+        }
 
         model = modelInterpolator.interpolate( model, context, strict );
 
+        // second pass allows ${user.home} to work, if it needs to.
+        // [MNG-2339] ensure the system properties are still interpolated for backwards compat, but the model values must win
+        if ( config.getExecutionProperties() != null && !config.getExecutionProperties().isEmpty() )
+        {
+            context.putAll( config.getExecutionProperties() );
+        }
+
+        model = modelInterpolator.interpolate( model, context, strict );
+
+        // MNG-3482: Make sure depMgmt is interpolated before merging.
+        if ( !isSuperPom )
+        {
+            mergeManagedDependencies( model, config.getLocalRepository(), remoteRepositories );
+        }
+
         // interpolation is before injection, because interpolation is off-limits in the injected variables
         modelDefaultsInjector.injectDefaults( model );
 
@@ -976,13 +1070,31 @@ public class DefaultMavenProjectBuilder
             }
         }
 
-        project.setParent( parentProject );
-
         if ( parentProject != null )
         {
-            Artifact parentArtifact = artifactFactory.createParentArtifact( parentProject.getGroupId(),
-                                                                            parentProject.getArtifactId(),
-                                                                            parentProject.getVersion() );
+            String cacheKey = createCacheKey( parentProject.getGroupId(),
+                                              parentProject.getArtifactId(),
+                                              parentProject.getVersion() );
+
+            MavenProject processedParent = (MavenProject) processedProjectCache.get( cacheKey );
+            Artifact parentArtifact;
+
+            // yeah, this null check might be a bit paranoid, but better safe than sorry...
+            if ( processedParent != null )
+            {
+                project.setParent( processedParent );
+
+                parentArtifact = processedParent.getArtifact();
+            }
+            else
+            {
+                project.setParent( parentProject );
+
+                parentArtifact = artifactFactory.createParentArtifact( parentProject.getGroupId(),
+                                                                                parentProject.getArtifactId(),
+                                                                                parentProject.getVersion() );
+            }
+
             project.setParentArtifact( parentArtifact );
         }
 
@@ -1017,11 +1129,10 @@ public class DefaultMavenProjectBuilder
      */
     private MavenProject assembleLineage( Model model,
                                           LinkedList lineage,
-                                          ArtifactRepository localRepository,
+                                          ProjectBuilderConfiguration config,
                                           File projectDir,
                                           List parentSearchRepositories,
                                           Set aggregatedRemoteWagonRepositories,
-                                          ProfileManager externalProfileManager,
                                           boolean strict )
         throws ProjectBuildingException, InvalidRepositoryException
     {
@@ -1043,15 +1154,15 @@ public class DefaultMavenProjectBuilder
             }
         }
 
-        //TODO mkleint - use the (Container, Properties constructor to make system properties embeddable
+        ProfileManager externalProfileManager = config.getGlobalProfileManager();
         ProfileManager profileManager;
-        if ( externalProfileManager != null && externalProfileManager instanceof DefaultProfileManager )
+        if ( externalProfileManager != null )
         {
-            profileManager = new DefaultProfileManager( container,
-                                                        ( (DefaultProfileManager) externalProfileManager ).getSystemProperties() );
+            profileManager = new DefaultProfileManager( container, externalProfileManager.getRequestProperties() );
         }
         else
         {
+            //TODO mkleint - use the (Container, Properties constructor to make system properties embeddable
             profileManager = new DefaultProfileManager( container );
         }
 
@@ -1132,7 +1243,7 @@ public class DefaultMavenProjectBuilder
 
             // if we can't find a cached model matching the parent spec, then let's try to look on disk using
             // <relativePath/>
-            if ( model == null && projectDir != null && StringUtils.isNotEmpty( parentRelativePath ) )
+            if ( ( model == null ) && ( projectDir != null ) && StringUtils.isNotEmpty( parentRelativePath ) )
             {
                 parentDescriptor = new File( projectDir, parentRelativePath );
 
@@ -1178,18 +1289,18 @@ public class DefaultMavenProjectBuilder
                     }
                 }
 
-                if ( parentDescriptor != null && parentDescriptor.exists() )
+                if ( ( parentDescriptor != null ) && parentDescriptor.exists() )
                 {
                     Model candidateParent = readModel( projectId, parentDescriptor, strict );
 
                     String candidateParentGroupId = candidateParent.getGroupId();
-                    if ( candidateParentGroupId == null && candidateParent.getParent() != null )
+                    if ( ( candidateParentGroupId == null ) && ( candidateParent.getParent() != null ) )
                     {
                         candidateParentGroupId = candidateParent.getParent().getGroupId();
                     }
 
                     String candidateParentVersion = candidateParent.getVersion();
-                    if ( candidateParentVersion == null && candidateParent.getParent() != null )
+                    if ( ( candidateParentVersion == null ) && ( candidateParent.getParent() != null ) )
                     {
                         candidateParentVersion = candidateParent.getParent().getVersion();
                     }
@@ -1252,7 +1363,7 @@ public class DefaultMavenProjectBuilder
 
                 try
                 {
-                    model = findModelFromRepository( parentArtifact, remoteRepositories, localRepository, false );
+                    model = findModelFromRepository( parentArtifact, remoteRepositories, config.getLocalRepository(), false );
                 }
                 catch ( ProjectBuildingException e )
                 {
@@ -1261,7 +1372,7 @@ public class DefaultMavenProjectBuilder
                 }
             }
 
-            if ( model != null && !"pom".equals( model.getPackaging() ) )
+            if ( ( model != null ) && !"pom".equals( model.getPackaging() ) )
             {
                 throw new ProjectBuildingException( projectId, "Parent: " + model.getId() + " of project: " +
                     projectId + " has wrong packaging: " + model.getPackaging() + ". Must be 'pom'." );
@@ -1273,9 +1384,13 @@ public class DefaultMavenProjectBuilder
                 parentProjectDir = parentDescriptor.getParentFile();
             }
 
-            MavenProject parent = assembleLineage( model, lineage, localRepository, parentProjectDir,
-                                                   parentSearchRepositories, aggregatedRemoteWagonRepositories,
-                                                   externalProfileManager, strict );
+            MavenProject parent = assembleLineage( model,
+                                                   lineage,
+                                                   config,
+                                                   parentProjectDir,
+                                                   parentSearchRepositories,
+                                                   aggregatedRemoteWagonRepositories,
+                                                   strict );
 
             parent.setFile( parentDescriptor );
 
@@ -1289,6 +1404,67 @@ public class DefaultMavenProjectBuilder
         return project;
     }
 
+    private void mergeManagedDependencies(Model model, ArtifactRepository localRepository, List parentSearchRepositories)
+        throws ProjectBuildingException
+    {
+        DependencyManagement modelDepMgmt = model.getDependencyManagement();
+
+        if (modelDepMgmt != null)
+        {
+            Map depsMap = new TreeMap();
+            Iterator iter = modelDepMgmt.getDependencies().iterator();
+            boolean doInclude = false;
+            while (iter.hasNext())
+            {
+                Dependency dep = (Dependency) iter.next();
+                depsMap.put( dep.getManagementKey(), dep );
+                if ( dep.getType().equals( "pom" ) && Artifact.SCOPE_IMPORT.equals( dep.getScope() ) )
+                {
+                    doInclude = true;
+                }
+            }
+            Map newDeps = new TreeMap(depsMap);
+            iter = modelDepMgmt.getDependencies().iterator();
+            if (doInclude)
+            {
+                while (iter.hasNext())
+                {
+                    Dependency dep = (Dependency)iter.next();
+                    if ( dep.getType().equals( "pom" )
+                         && Artifact.SCOPE_IMPORT.equals( dep.getScope() ) )
+                    {
+                        Artifact artifact = artifactFactory.createProjectArtifact( dep.getGroupId(), dep.getArtifactId(),
+                                                                                  dep.getVersion(), dep.getScope() );
+                        MavenProject project = buildFromRepository(artifact, parentSearchRepositories, localRepository, false);
+
+                        DependencyManagement depMgmt = project.getDependencyManagement();
+
+                        if (depMgmt != null)
+                        {
+                            if ( getLogger().isDebugEnabled() )
+                            {
+                                getLogger().debug( "Importing managed dependencies for " + dep.toString() );
+                            }
+
+                            for ( Iterator it = depMgmt.getDependencies().iterator(); it.hasNext(); )
+                            {
+                                Dependency includedDep = (Dependency) it.next();
+                                String key = includedDep.getManagementKey();
+                                if (!newDeps.containsKey(key))
+                                {
+                                    newDeps.put( includedDep.getManagementKey(), includedDep );
+                                }
+                            }
+                            newDeps.remove(dep.getManagementKey());
+                        }
+                    }
+                }
+                List deps = new ArrayList(newDeps.values());
+                modelDepMgmt.setDependencies(deps);
+            }
+        }
+    }
+
     private List injectActiveProfiles( ProfileManager profileManager,
                                        Model model )
         throws ProjectBuildingException
@@ -1337,7 +1513,7 @@ public class DefaultMavenProjectBuilder
                 {
                     List active = root.getActiveProfiles();
 
-                    if ( active != null && !active.isEmpty() )
+                    if ( ( active != null ) && !active.isEmpty() )
                     {
                         profileManager.explicitlyActivate( root.getActiveProfiles() );
                     }
@@ -1599,6 +1775,7 @@ public class DefaultMavenProjectBuilder
     public void contextualize( Context context )
         throws ContextException
     {
-        this.container = (PlexusContainer) context.get( PlexusConstants.PLEXUS_KEY );
+        container = (PlexusContainer) context.get( PlexusConstants.PLEXUS_KEY );
     }
+
 }
